      PROGRAM TRISTM16
C********************** TRISTM ********************************
C***
C***  THIS PROGRAM IS DESIGNED TO INVERT MUNSELL COORDINATES
C***  (HUE, VALUE, & SATURATION) DERIVED USING THE PROGRAM
C***  MUNSEL.
C***
C***  THESE DERIVED VALUES ARE REFERRED TO AS TRISTIMULUS
C***  VALUES.
C***
C***  THE PROGRAM ASSUMES A COLOR ADDITIVE SYSTEM.
C***
C***  COLOR SUBTRACTIVE PRIMARIES     COLOR ADDITIVE PRIMARIES
C***          CYAN --------------------------- RED
C***          YELLOW ------------------------- BLUE
C***          MAGENTA ------------------------ GREEN
C***
C***  EQUATIONS WERE DERIVED BY DON SAWATZKY AND THE PROGRAM WAS
C***  WRITTEN BY GARY RAINES.
C***
C***
C***  8 or 16 bit image files input, 16bit image files output
C***
C***
C***  LINK WITH DISKIO, UNPACK, REMLOG, DATE, TIME, EXIT
C***
C*****************************************************************
      INTEGER*2 IRED(8192),IGRN(8192),IBLU(8192)
      INTEGER*2 IHUE[FAR](8192)
      INTEGER*2 ISAT[FAR](8192)
      INTEGER*2 IVAL[FAR](8192)
      INTEGER*4 FHUE(128),FSAT(128),FVAL(128)
      INTEGER*4 FRED(128),FGRN(128),FBLU(128)
      INTEGER*4 TTY,TTYO,IPR
      INTEGER*4 LINES,I
      REAL*8 K2,K6,K7,K3
      REAL*8 PIE, HUE, SAT, V1, BLU, GRN, RED, B1, X1
      DATA FRED,FGRN,FBLU/128*0,128*0,128*0/
      DATA FHUE,FSAT,FVAL/128*0,128*0,128*0/
      DATA TTY,TTYO/5,6/,IPR/4/
      K2=SQRT(2.)/2.
      K6=SQRT(6.)/6.
      K3=SQRT(3.)/3.
      K7=SQRT(6.)/3.
      LINES=0
      PIE=3.14159265358979323846
      CALL REMLOG('********************* TRISTM *********************')
      WRITE(TTYO,5)
5     FORMAT(' HUE, SATURATION, AND VALUE INPUT FILES.')
      WRITE(IPR,10)
10    FORMAT(' OPEN HUE FILE')
      CALL DISKIO(0,11,IHUE,FHUE)
      DO 7  J=2,5
      FSAT(J)=FHUE(J)
7     FVAL(J)=FHUE(J)
      WRITE(IPR,20)
20    FORMAT(' OPEN SATURATION FILE.')
      CALL DISKIO(0,12,ISAT,FSAT)
      WRITE(IPR,30)
30    FORMAT(' OPEN VALUE FILE.')
      CALL DISKIO(0,13,IVAL,FVAL)
C***
C***  OPEN OUTPUT FILES FOR TRISTIMULUS VALUES
C***
      FRED(1)=16
      FRED(2)=FHUE(2)
      FGRN(1)=16
      FGRN(2)=FHUE(2)
      FBLU(1)=16
      FBLU(2)=FHUE(2)
      WRITE(TTYO,34)
34    FORMAT('  RED, GREEN, AND BLUE OUTPUT FILES.')
      WRITE(IPR,35)
35    FORMAT('  OPEN OUTPUT RED FILE,  (CYAN).')
      CALL DISKIO(0,14,IRED,FRED)
      WRITE(IPR,36)
36    FORMAT('  OPEN OUTPUT GREEN FILE,  (MAGENTA).')
      CALL DISKIO(0,15,IGRN,FGRN)
      WRITE(IPR,37)
37    FORMAT('  OPEN OUTPUT BLUE FILE,   (YELLOW)')
      CALL DISKIO(0,16,IBLU,FBLU)
C***
C***  READ A SCANLINE TO OPERATE ON
C***
40    CALL DISKIO(9,11,IHUE,FHUE)
         IF(FHUE(1).LT.0) GOTO 999
         IF(FHUE(25).EQ.8) CALL UNPACK(IHUE,FHUE)
      CALL DISKIO(9,12,ISAT,FSAT)
         IF(FSAT(1).LT.0) GOTO 999
         IF(FSAT(25).EQ.8) CALL UNPACK(ISAT,FSAT)      
      CALL DISKIO(9,13,IVAL,FVAL)
         IF(FVAL(1).LT.0) GOTO 999
         IF(FVAL(25).EQ.8) CALL UNPACK(ISAT,FSAT)
C***      
      I=1
45    HUE=IHUE(I)
      SAT=ISAT(I)
      V1=IVAL(I)
C***
C***  CALCULATE THE INVERSE OF THE MUNSELL COORDINATES
C***
      HUE=HUE*PIE/180.
      B1=SAT*COS(HUE)
      X1=SAT*SIN(HUE)
      BLU=K7*B1+K3*V1
      GRN=K2*X1+K3*V1-K6*B1
      RED=K3*V1-K2*X1-K6*B1
      IRED(I)=RED+0.5
      IGRN(I)=GRN+0.5
      IBLU(I)=BLU+0.5
      IF(IRED(I).GT.255) IRED(I)=255
      IF(IGRN(I).GT.255) IGRN(I)=255
      IF(IBLU(I).GT.255) IBLU(I)=255
      IF(IRED(I).LT.0) IRED(I)=0
      IF(IGRN(I).LT.0) IGRN(I)=0
      IF(IBLU(I).LT.0) IBLU(I)=0
      I=I+1
      IF(I.LE.FHUE(2)) GOTO 45
C***
C***  WRITE A COLOR CLASSIFIED SCAN LINE
C***
      CALL DISKIO(10,14,IRED,FRED)
      CALL DISKIO(10,15,IGRN,FGRN)
      CALL DISKIO(10,16,IBLU,FBLU)
      LINES=LINES+1
      IF(LINES.LT.FHUE(3)) GOTO 40
C***
C***  CLOSE FILES
C***
999   CALL DISKIO(6,11,IHUE,FHUE)
      CALL DISKIO(6,12,ISAT,FSAT)
      CALL DISKIO(6,13,IVAL,FVAL)
      CALL DISKIO(6,14,IRED,FRED)
      CALL DISKIO(6,15,IGRN,FGRN)
      CALL DISKIO(6,16,IBLU,FBLU)
      STOP
      END
      
